# kalah

## Setup
To get started, run `stack build`.

In the project, we implemented several Kalah AIs:
- SeqMM: Sequential min-max.
- SeqAB: Sequential alpha-beta pruning.
- ParThenMM: Parallel min-max, then sequential min-max.
- ParThenAB: Parallel min-max, then sequential alpha-beta pruning.
- YoungBros: Young Brothers Wait Concept.

You can interact with them through these modes that we implemented:
- PvE (Player vs AI)
- EvE (AI vs AI)
- OneMove
- Benchmark

## PvE Mode
In PvE mode, you can play Kalah against an AI.

To run it, you can use the following command:
```
stack exec kalah-exe pve [mode] [params]
```
`[mode]` refers to the Kalah AI that you want to play against. This can be one
of `seq_mm`, `seq_ab`, `par_then_mm`, `par_then_ab`, `young_bros`.

For `seq_mm` and `seq_ab`, you must specify `[params]` as
`"(search_depth_limit)"`.
For `par_then_mm`, `par_then_ab`, and `young_bros`, you must specify `[params]`
as `"(search_depth_limit, parallelism_depth_limit)"`.

E.g.
```
stack exec kalah-exe pve seq_mm "(9)"
stack exec kalah-exe pve par_then_ab "(8,2)" -- +RTS -N4 -ls
```

## EvE (AI vs AI) Mode
In EvE mode, you can watch two AIs play against each other.

To run it, you can use the following command:
```
stack exec kalah-exe eve [mode1] [params1] [mode2] [params2]
```

`[mode1]` and `[params1]` refer to the mode and parameters of the first AI.
`[mode2]` and `[params2]` refer to the mode and parameters of the second AI.

E.g.
```
stack exec kalah-exe eve seq_mm "(4)" par_then_ab "(7,2)" -- +RTS -N4 -ls
stack exec kalah-exe eve seq_ab "(5)" seq_ab "(7)"
```

## OneMove Mode
OneMove mode allows you to run the AI on the initial Kalah board once.

This was mainly for debugging purposes.

E.g.
```
stack exec kalah-exe onemove seq_ab "(8)"
stack exec kalah-exe onemove par_then_ab "(7,2)" -- +RTS -N4 -ls
```

## Benchmark Mode

We wrote several benchmark tests using the Criterion package.

For `SeqMM` and `SeqAB`, we wrote tests that benchmarks them by running 
them on the initial Kalah board many times with a search depth limit of 9.

For the other 3 AIs, we did the same thing, but also have tests varying
the parallelism depth limit from 1 to 9.

Commands to run them:
```
stack exec kalah-exe benchmark seq_mm
stack exec kalah-exe benchmark seq_ab
stack exec kalah-exe benchmark par_then_mm -- +RTS -N4 -ls
stack exec kalah-exe benchmark par_then_ab -- +RTS -N4 -ls
stack exec kalah-exe benchmark young_bros -- +RTS -N4 -ls
```

## Benchmark Scripts

For the purpose of collecting data, we wrote these bash scripts:
```
run_runtime_benchmarks.sh
run_spark_benchmarks.sh
```
They are located in the top-level directory of the project.

These scripts produce a bunch of `.txt` files, so make sure to run `rm *.txt`
after using them!

## Organization of project:

- `app/Main.hs` - Parses cmd line arguments and calls appropriate functions.
- `src/Benchmark.hs` - Contains benchmark tests.
- `src/Kalah.hs` - Contains Kalah-specific data structures and functions.
- `src/Minmax.hs` - Contains Kalah AIs.
- `src/Play.hs` - Contains code for PvE, EvE, and OneMove modes.
- `test/Spec.hs` - Contains unit tests.

## Testing
The unit tests in test/Spec.hs can be run via `stack test`
