module Main (main) where

import Text.Read
import MinMax
import Play
import Benchmark
import System.Environment
import System.Exit

main :: IO ()
main = do
  args <- getArgs
  case args of
    ["pve", mode, params] -> do
      minMaxConfig <- parseModeAndParams mode params
      pVsAI minMaxConfig
    ["eve", mode1, params1, mode2, params2] -> do
      minMaxConfig1 <- parseModeAndParams mode1 params1
      minMaxConfig2 <- parseModeAndParams mode2 params2
      aIVsAI minMaxConfig1 minMaxConfig2
      exitFailure
    ["onemove", mode, params] -> do
      minMaxConfig <- parseModeAndParams mode params
      runOneMove minMaxConfig
    ("benchmark":benchmarkName) -> do
      runBench benchmarkName
    _ -> do
      pn <- getProgName
      let usage = "Usage: " ++ pn ++ " pve AI_1_mode AI_1_params\n\
        \Usage: " ++ pn ++ " eve AI_1_mode AI_1_params \
        \AI_2_mode AI_2_params\n\
        \Usage: " ++ pn ++ " onemove AI_1_mode AI_1_params"
      die $ usage

parseModeAndParams :: String -> String -> IO MinMaxConfig
parseModeAndParams mode' params' = do
  case mode' of
    "seq_mm" -> do
      depth <- parseOne params'
      return $ MinMaxConfig SeqMM depth (-1)
    "seq_ab" -> do
      depth <- parseOne params'
      return $ MinMaxConfig SeqAB depth (-1)
    "par_then_mm" -> do
      (depth, parDepth) <- parseTwo params'
      return $ MinMaxConfig ParThenMM depth parDepth
    "par_then_ab" -> do
      (depth, parDepth) <- parseTwo params'
      return $ MinMaxConfig ParThenAB depth parDepth
    "young_bros" -> do
      (depth, parDepth) <- parseTwo params'
      return $ MinMaxConfig YoungBros depth parDepth
    _ -> do
      die $ "Error: AI Mode must be one of \
        \[seq_mm, seq_ab, par_then_mm, par_then_ab, young_bros]"

parseOne :: String -> IO Int
parseOne params' = do
  case readMaybe params' of
    Just arg1 -> do
      if arg1 < 0 then
        die $ "Error: search_depth_limit must be non-negative"
      else return arg1
    Nothing -> do
      die $ "Error: AI params must be specified as \"(search_depth_limit)\""

parseTwo :: String -> IO (Int, Int)
parseTwo params' = do
  case readMaybe params' of
    Just (arg1,arg2) -> do
      if arg1 < 0 || arg2 < 0 then
        die $ "Error: search_depth_limit and parallelism_depth_limit must \
              \be non-negative"
      else return (arg1,arg2)
    Nothing -> do
      die $ "Error: AI params must be specified as \"(search_depth_limit,\
        \parallelism_depth_limit)\""
