module Benchmark
    ( runBench
    ) where

import Criterion.Main
import MinMax
import Kalah
import System.Environment

-- Runs tests on our Kalah AI implementations with search depth limit 9
runBench :: [String] -> IO ()
runBench benchmarkName = withArgs benchmarkName tests
  where genBench mode dep = [bench (show dep) $ doBench mode dep]
        doBench mode dep = nf doBench' $ MinMaxConfig mode dep (-1)
        genBenchPar mode dep range =
          [bench (show i) $ doBenchPar mode dep i | i <- range]
        doBenchPar mode dep parDep =
          nf doBench' $ MinMaxConfig mode dep parDep
        doBench' minMaxConfig = computeAIMove minMaxConfig initState
        tests = 
          defaultMain [
            bgroup "seq_mm" $ genBench SeqMM 9
          , bgroup "seq_ab" $ genBench SeqAB 9
          , bgroup "par_then_mm" $ genBenchPar ParThenMM 9 [1..9]
          , bgroup "par_then_ab" $ genBenchPar ParThenAB 9 [1..9]
          , bgroup "young_bros" $ genBenchPar YoungBros 9 [1..9]
          ] 
