module Play
    ( pVsAI,
      aIVsAI,
      runOneMove
    ) where

import Kalah
import Text.Read
import MinMax

-- PvE mode
pVsAI :: MinMaxConfig -> IO ()
pVsAI minMaxConfig = do
  putStrLn "Initializing Board..."
  let s = initState
  putStrLn $ getBoardString s
  let computeAIMove' = computeAIMove minMaxConfig
  doPVsAI computeAIMove' s

-- PvE mode helper
doPVsAI :: (State -> Int) -> State -> IO ()
doPVsAI computeAI1Move s = do
  if isTerm s
    then do
      let winner = whoWon s
      putStrLn winner
      let score = getScore s
      putStrLn score
      return ()
  else do
    putStrLn $ "Player " ++ (show $ whoseTurn s) ++ "'s turn."
    if getIsP0Turn s
      then do
        s' <- makePlayerTurn s
        printAndDoPlay s'
    else do
      s' <- makeAITurn computeAI1Move s
      printAndDoPlay s'
  where printAndDoPlay s' = do
          putStrLn $ getBoardString s'
          putStrLn "\n"
          doPVsAI computeAI1Move s'

-- EvE mode
aIVsAI :: MinMaxConfig -> MinMaxConfig -> IO ()
aIVsAI minMaxConfig1 minMaxConfig2 = do
  putStrLn "Initializing Board..."
  let s = initState
  putStrLn $ getBoardString s
  let computeAIMove1 = computeAIMove minMaxConfig1
  let computeAIMove2 = computeAIMove minMaxConfig2
  doAIVsAI computeAIMove1 computeAIMove2 s

-- EvE mode helper
doAIVsAI :: (State -> Int) -> (State -> Int) -> State -> IO ()
doAIVsAI computeAIMove1 computeAIMove2 s = do
  if isTerm s
    then do
      let winner = whoWon s
      putStrLn winner
      let score = getScore s
      putStrLn score
  else do
    putStrLn $ "Player " ++ (show $ whoseTurn s) ++ "'s turn."
    if getIsP0Turn s
      then do
        s' <- makeAITurn computeAIMove1 s
        printAndDoPlay s'
    else do
      s' <- makeAITurn computeAIMove2 s
      printAndDoPlay s'
  where printAndDoPlay s' = do
          putStrLn $ getBoardString s' ++ "\n"
          doAIVsAI computeAIMove1 computeAIMove2 s'

makePlayerTurn :: State -> IO State
makePlayerTurn s = do
  let validMoves = map (1+) $ getValidMoves s
  putStrLn $ "Valid Moves are: " ++ (show validMoves)
  line <- getLine
  case readMaybe line of
    Nothing -> do
        putStrLn "Invalid move."
        makePlayerTurn s
    Just a -> do
      let maybeS' = makeMove s (a - 1)
      case maybeS' of
        Just s' -> do
          return s'
        Nothing -> do
          putStrLn "Invalid move."
          makePlayerTurn s

makeAITurn :: (State -> Int) -> State -> IO State
makeAITurn computeAIMove' s = do
  let a = computeAIMove' s
  let maybeS' = makeMove s a
  case maybeS' of
    Just s' -> do
      putStrLn $ "AI picked " ++ (show $ a + 1) ++ "."
      return s'
    Nothing -> do
      return $ error $ "AI tried to make invalid move."        

-- Just run AI on initial state
runOneMove :: MinMaxConfig -> IO ()
runOneMove minMaxConfig = do
  putStrLn "Initializing Board..."
  let move = computeAIMove minMaxConfig initState
  putStrLn $ "Optimal move is " ++ show move
